package com.biganto.visual.roompark.domain.use_case

import com.biganto.visual.roompark.data.data_provider.featuresVariants
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.UserEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.DealContract
import com.biganto.visual.roompark.domain.contract.FlatPlanContract
import io.reactivex.Observable
import io.reactivex.rxkotlin.Observables
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 20.04.2020.
 */

class PlanTypesUseCase @Inject constructor(
    private val planContract: FlatPlanContract,
    private val auth:AuthContract,
    private val dealContract: DealContract
) {

    private fun fetchUserEstates(user:UserEntity): Observable<List<EstateEntity>> =
                Observables.zip(
                    dealContract.fetchDeals(user)
                        .map { it.map {deal -> deal.estate as EstateEntity } }
                    ,dealContract.fetchFavorites(user))
                    {t1,t2 -> t1+t2}

    private val fetchAllPlanTypes =
        auth.currentUser()
            .flatMap {user ->
                fetchUserEstates(user)
                    .onErrorReturn { arrayListOf() }
                    .map { apiList ->
                        val list =
                            user.deals?.map { it.estate as EstateEntity }?.toMutableList()
                                ?: mutableListOf()
                        list.addAll(user.estates?.map { it as EstateEntity } ?: arrayListOf())
                        (list+apiList).distinctBy { it.id }
                    }
            }



    fun fetchNotDownloadedPlansSizes(): Observable<Int> =
        fetchAllPlanTypes
            .flatMapIterable { it }
            .flatMap {
                planContract.getPlanTypes(it.id)
                    .map { plantypes -> plantypes.flatMap { plan -> plan.featuresVariants } }
            }
            .map { list -> list.sumBy { if (planContract.getPlanFile(it).exists()) 0 else 1 } }
            .scan { t1: Int, t2: Int -> t1 + t2 }

    fun downloadAllPlanTypes(cancellationToken: DownloadUseCase.CancellationToken): Observable<DownloadProgress> {
        var listSize = 0
        return fetchAllPlanTypes
            .flatMapSingle {
                Observable.fromIterable(it)
                    .flatMap { estateEntity ->
                        planContract.getPlanTypes(estateEntity.id)
                            .map { plantypes -> plantypes.flatMap { plan -> plan.featuresVariants } }
                    }
                    .toList()
            }
            .map { it.flatten() }
            .filter { !cancellationToken.isCancelled }
            .flatMap { list ->
                planContract.getPlansObservable(list,cancellationToken)
                    .map { 1 }
                    .scan(0,
                        { t1, t2 -> t1 + t2 })
                    .map { completed ->
                        DownloadProgress(completed, list.size)
                    }
            }
    }
}

//    private fun downloadPlan(
//        list: PlanFeaturesVariant
//        , cancellationToken: DownloadUseCase.CancellationToken
//    )
//            : Observable<DownloadProgress> {
//
//        val avaliablethread = true
//
//        return  Observable.fromIterable(list)
//
//
//            .delay(100,TimeUnit.MILLISECONDS)
//            .doOnNext { Timber.w("emmited") }
//            .delay { variant ->
//                planContract.getPlan(variant)}
//            .filter { !cancellationToken.isCancelled }
//
//    }





data class DownloadProgress(
    val currentProgress:Int,
    val totalProgress:Int
)